figure figcaption p {
    margin: 0em 0em 3em 0em !important;  /* top, right, bottom, left */
    font-size: small;
    text-align: center;
}

/* Full screen modal */
#image-modal {
    display: none;
    position: fixed;
    z-index: 9999;
    left: 0; top: 0;
    width: 100%; height: 100%;
    background-color: rgba(0,0,0,0.8);
    justify-content: center;
    align-items: center;
}

/* Real size picture */
#image-modal img {
    max-width: none;
    max-height: 90vh;  /* limite hauteur écran */
    box-shadow: 0 0 10px #000;
}

/* Model displayed */
#image-modal.show {
    display: flex;
    cursor: pointer;
}
